print("import: 'wtforms'")
import wtforms

print("import: 'wtforms.csrf'")
import wtforms.csrf

print("import: 'wtforms.ext'")
import wtforms.ext

print("import: 'wtforms.ext.appengine'")
import wtforms.ext.appengine

print("import: 'wtforms.ext.csrf'")
import wtforms.ext.csrf

print("import: 'wtforms.ext.dateutil'")
import wtforms.ext.dateutil

print("import: 'wtforms.ext.django'")
import wtforms.ext.django

print("import: 'wtforms.ext.django.templatetags'")
import wtforms.ext.django.templatetags

print("import: 'wtforms.ext.i18n'")
import wtforms.ext.i18n

print("import: 'wtforms.ext.sqlalchemy'")
import wtforms.ext.sqlalchemy

print("import: 'wtforms.fields'")
import wtforms.fields

print("import: 'wtforms.widgets'")
import wtforms.widgets

